@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.0.") or
-    raise "ruby lib version (3.0.4) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.0.5) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.0/powerpc-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "0"
-  CONFIG["TEENY"] = "4"
-  CONFIG["PATCHLEVEL"] = "208"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.4/bin/install -c'
+  CONFIG["TEENY"] = "5"
+  CONFIG["PATCHLEVEL"] = "211"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.5/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)30"
@@ -44,7 +44,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=30' '--with-soname=ruby30' '--with-ruby-version=minor' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=30' '--with-soname=ruby30' '--with-ruby-version=3.0' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CXXFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -63,8 +63,8 @@ module RbConfig
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
   CONFIG["libdirname"] = "libdir"
   CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
-  CONFIG["RUBY_LIB_VERSION"] = ""
-  CONFIG["RUBY_LIB_VERSION_STYLE"] = "2\t/* minor */"
+  CONFIG["RUBY_LIB_VERSION"] = "3.0"
+  CONFIG["RUBY_LIB_VERSION_STYLE"] = ""
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
   CONFIG["rubysitearchprefix"] = "$(rubylibprefix)/$(sitearch)"
@@ -121,6 +121,7 @@ module RbConfig
   CONFIG["RPATHFLAG"] = " -Wl,-rpath,%1$-s"
   CONFIG["LIBPATHFLAG"] = " -L%1$-s"
   CONFIG["LINK_SO"] = ""
+  CONFIG["ADDITIONAL_DLDFLAGS"] = ""
   CONFIG["ASMEXT"] = "S"
   CONFIG["LIBEXT"] = "a"
   CONFIG["DLEXT2"] = ""
@@ -132,6 +133,8 @@ module RbConfig
   CONFIG["ARCH_FLAG"] = ""
   CONFIG["DLDFLAGS"] = "-L/usr/local/lib"
   CONFIG["ALLOCA"] = ""
+  CONFIG["EGREP"] = "/usr/bin/egrep"
+  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["dsymutil"] = ""
   CONFIG["codesign"] = ""
   CONFIG["POSTLINK"] = ":"
@@ -147,9 +150,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.4/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.4/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.4/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.5/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -157,7 +160,7 @@ module RbConfig
   CONFIG["ASFLAGS"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 13.0.0\nTarget: powerpc-unknown-openbsd7.2\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 13.0.0\nTarget: powerpc-unknown-openbsd7.3\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["MJIT_CC"] = "/usr/bin/cc -fdeclspec"
   CONFIG["CSRCFLAG"] = ""
@@ -166,14 +169,12 @@ module RbConfig
   CONFIG["CPPOUTFILE"] = "-o conftest.i"
   CONFIG["GNU_LD"] = "no"
   CONFIG["GCC"] = "yes"
-  CONFIG["EGREP"] = "/usr/bin/egrep"
-  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["CPP"] = "cc -E"
-  CONFIG["CXXFLAGS"] = "-O2 -pipe"
+  CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
   CONFIG["OBJEXT"] = "o"
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E -Wl,--no-as-needed"
-  CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
+  CONFIG["CXXFLAGS"] = "-O2 -pipe"
   CONFIG["STRIP"] = "strip -S -x"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["OBJDUMP"] = "objdump"
@@ -184,18 +185,18 @@ module RbConfig
   CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc -fdeclspec"
-  CONFIG["target_os"] = "openbsd7.2"
+  CONFIG["target_os"] = "openbsd7.3"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "powerpc"
-  CONFIG["target"] = "powerpc-unknown-openbsd7.2"
-  CONFIG["host_os"] = "openbsd7.2"
+  CONFIG["target"] = "powerpc-unknown-openbsd7.3"
+  CONFIG["host_os"] = "openbsd7.3"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "powerpc"
-  CONFIG["host"] = "powerpc-unknown-openbsd7.2"
-  CONFIG["build_os"] = "openbsd7.2"
+  CONFIG["host"] = "powerpc-unknown-openbsd7.3"
+  CONFIG["build_os"] = "openbsd7.3"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "powerpc"
-  CONFIG["build"] = "powerpc-unknown-openbsd7.2"
+  CONFIG["build"] = "powerpc-unknown-openbsd7.3"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
