@@ -22,7 +22,7 @@
 #   ruby -run -e touch -- [OPTION] FILE
 #   ruby -run -e wait_writable -- [OPTION] FILE
 #   ruby -run -e mkmf -- [OPTION] EXTNAME [OPTION]
-#   ruby -run -e httpd -- [OPTION] DocumentRoot
+#   ruby -run -e httpd -- [OPTION] [DocumentRoot]
 #   ruby -run -e help [COMMAND]
 
 require "fileutils"
@@ -88,7 +88,7 @@ def cp
     options[:preserve] = true if options.delete :p
     dest = argv.pop
     argv = argv[0] if argv.size == 1
-    FileUtils.send cmd, argv, dest, **options
+    FileUtils.__send__ cmd, argv, dest, **options
   end
 end
 
@@ -109,7 +109,7 @@ def ln
     options[:force] = true if options.delete :f
     dest = argv.pop
     argv = argv[0] if argv.size == 1
-    FileUtils.send cmd, argv, dest, **options
+    FileUtils.__send__ cmd, argv, dest, **options
   end
 end
 
@@ -144,7 +144,7 @@ def rm
     cmd = "rm"
     cmd += "_r" if options.delete :r
     options[:force] = true if options.delete :f
-    FileUtils.send cmd, argv, **options
+    FileUtils.__send__ cmd, argv, **options
   end
 end
 
@@ -161,7 +161,7 @@ def mkdir
   setup("p") do |argv, options|
     cmd = "mkdir"
     cmd += "_p" if options.delete :p
-    FileUtils.send cmd, argv, **options
+    FileUtils.__send__ cmd, argv, **options
   end
 end
 
@@ -304,7 +304,7 @@ end
 ##
 # Run WEBrick HTTP server.
 #
-#   ruby -run -e httpd -- [OPTION] DocumentRoot
+#   ruby -run -e httpd -- [OPTION] [DocumentRoot]
 #
 #   --bind-address=ADDR         address to bind
 #   --port=NUM                  listening port number
@@ -326,7 +326,11 @@ def httpd
         "ServerName=NAME", "ServerSoftware=NAME",
         "SSLCertificate=CERT", "SSLPrivateKey=KEY") do
     |argv, options|
-    require 'webrick'
+    begin
+      require 'webrick'
+    rescue LoadError
+      abort "webrick is not found. You may need to `gem install webrick` to install webrick."
+    end
     opt = options[:RequestTimeout] and options[:RequestTimeout] = opt.to_i
     [:Port, :MaxClients].each do |name|
       opt = options[name] and (options[name] = Integer(opt)) rescue nil
