@@ -73,9 +73,9 @@ module TypeProf
       else
         ty = Type::Instance.new(recv)
       end
-      meths = scratch.get_method(recv, false, :initialize)
+      meths = scratch.get_method(recv, false, false, :initialize)
       meths.flat_map do |meth|
-        meth.do_send(ty, :initialize, aargs, ep, env, scratch) do |ret_ty, ep, env|
+        meth.do_send(ty, :initialize, aargs, ep, env, scratch) do |_ret_ty, ep, env|
           ctn[ty, ep, env]
         end
       end
@@ -104,7 +104,7 @@ module TypeProf
         sym = get_sym("respond_to?", aargs.lead_tys[0], ep, scratch)
         if sym
           klass, singleton = recv.method_dispatch_info
-          if scratch.get_method(klass, singleton, sym)
+          if scratch.get_method(klass, singleton, false, sym)
             true_val = Type::Instance.new(Type::Builtin[:true])
             ctn[true_val, ep, env]
           else
@@ -158,7 +158,43 @@ module TypeProf
       naargs = ActualArguments.new([recv], nil, {}, Type.nil)
       nrecv = recv
       nrecv = nrecv.base_type if nrecv.is_a?(Type::ContainerType)
-      scratch.do_invoke_block(aargs.blk_ty, naargs, ep, env, replace_recv_ty: nrecv) do |_ret_ty, ep|
+      scratch.do_invoke_block(aargs.blk_ty, naargs, ep, env, replace_recv_ty: nrecv) do |ret_ty, ep|
+        ctn[ret_ty, ep, env]
+      end
+    end
+
+    def module_eqq(recv, mid, aargs, ep, env, scratch, &ctn)
+      if aargs.lead_tys.size == 1
+        aargs.lead_tys[0].each_child do |aarg|
+          aarg = aarg.base_type if aarg.is_a?(Type::Symbol) # XXX
+          if aarg.is_a?(Type::Instance)
+            if aarg.klass == recv # XXX: inheritance
+              true_val = Type::Instance.new(Type::Builtin[:true])
+              ctn[true_val, ep, env]
+            else
+              false_val = Type::Instance.new(Type::Builtin[:false])
+              ctn[false_val, ep, env]
+            end
+          else
+            ctn[Type.bool, ep, env]
+          end
+        end
+      else
+        ctn[Type.bool, ep, env]
+      end
+    end
+
+    def object_module_eval(recv, mid, aargs, ep, env, scratch, &ctn)
+      if aargs.lead_tys.size >= 1
+        scratch.warn(ep, "class_eval with arguments is ignored")
+        ctn[Type.any, ep, env]
+        return
+      end
+      naargs = ActualArguments.new([recv], nil, {}, Type.nil)
+      nrecv = recv
+      nrecv = nrecv.base_type if nrecv.is_a?(Type::ContainerType)
+      ncref = ep.ctx.cref.extend(nrecv, true)
+      scratch.do_invoke_block(aargs.blk_ty, naargs, ep, env, replace_recv_ty: nrecv, replace_cref: ncref) do |_ret_ty, ep|
         ctn[recv, ep, env]
       end
     end
@@ -176,7 +212,7 @@ module TypeProf
       end
 
       elem_ty = Type.bot
-      enum_for_blk = CustomBlock.new(ep, mid) do |aargs, caller_ep, caller_env, scratch, replace_recv_ty:, &blk_ctn|
+      enum_for_blk = CustomBlock.new(ep, mid) do |aargs, caller_ep, caller_env, scratch, replace_recv_ty:, replace_cref:, &blk_ctn|
         if aargs.lead_tys.size >= 1
           elem_ty = elem_ty.union(aargs.lead_tys[0])
         else
@@ -235,9 +271,12 @@ module TypeProf
         return ctn[Type.any, ep, env]
       end
 
+      # support multiple arguments: include M1, M2
       arg = aargs.lead_tys[0]
       arg.each_child do |arg|
         if arg.is_a?(Type::Class)
+          aargs = ActualArguments.new([recv], nil, {}, Type.nil)
+          scratch.do_send(arg, :included, aargs, ep, env) {|_ret_ty, _ep| }
           scratch.mix_module(:after, recv, arg, nil, ep.ctx.cref.singleton, ep)
         end
       end
@@ -259,6 +298,8 @@ module TypeProf
       arg = aargs.lead_tys[0]
       arg.each_child do |arg|
         if arg.is_a?(Type::Class)
+          aargs = ActualArguments.new([recv], nil, {}, Type.nil)
+          scratch.do_send(arg, :extended, aargs, ep, env) {|_ret_ty, _ep| }
           # if ep.ctx.cref.singleton is true, the meta-meta level is ignored. Should we warn?
           scratch.mix_module(:after, recv, arg, nil, true, ep)
         end
@@ -293,7 +334,7 @@ module TypeProf
       else
         aargs.lead_tys.each do |aarg|
           sym = get_sym("module_function", aarg, ep, scratch) or next
-          meths = scratch.get_method(recv, false, sym)
+          meths = scratch.get_method(recv, false, false, sym)
           meths.each do |mdef|
             scratch.add_method(recv, sym, true, mdef)
           end
@@ -309,7 +350,7 @@ module TypeProf
         if recv.is_a?(Type::Class)
           aargs.lead_tys.each do |aarg|
             sym = get_sym("public", aarg, ep, scratch) or next
-            meths = scratch.get_method(recv, false, sym)
+            meths = scratch.get_method(recv, false, false, sym)
             next unless meths
             meths.each do |mdef|
               mdef.pub_meth = true if mdef.respond_to?(:pub_meth=)
@@ -329,7 +370,7 @@ module TypeProf
         if recv.is_a?(Type::Class)
           aargs.lead_tys.each do |aarg|
             sym = get_sym("private", aarg, ep, scratch) or next
-            meths = scratch.get_method(recv, false, sym)
+            meths = scratch.get_method(recv, false, false, sym)
             next unless meths
             meths.each do |mdef|
               mdef.pub_meth = false if mdef.respond_to?(:pub_meth=)
@@ -594,9 +635,6 @@ module TypeProf
     end
 
     def self.file_require(feature, scratch)
-      return :done, :false if scratch.loaded_features[feature]
-      scratch.loaded_features[feature] = true
-
       # XXX: dynamic RBS load is really needed??  Another idea:
       #
       # * RBS should be loaded in advance of analysis
@@ -612,7 +650,11 @@ module TypeProf
 
       begin
         filetype, path = $LOAD_PATH.resolve_feature_path(feature)
+
         if filetype == :rb
+          return :done, :false if scratch.loaded_files[path]
+          scratch.loaded_files[path] = true
+
           return :do, path if File.readable?(path)
 
           return :error, "failed to load: #{ path }"
@@ -673,13 +715,14 @@ module TypeProf
           return ctn[Type.any, ep, env]
         end
 
-        if scratch.loaded_features[feature]
+        path = File.join(File.dirname(ep.ctx.iseq.absolute_path), feature) + ".rb" # XXX
+
+        if scratch.loaded_files[path]
           result = Type::Instance.new(Type::Builtin[:false])
           return ctn[result, ep, env]
         end
-        scratch.loaded_features[feature] = true
+        scratch.loaded_files[path] = true
 
-        path = File.join(File.dirname(ep.ctx.iseq.path), feature) + ".rb" # XXX
         return Builtin.file_load(path, ep, env, scratch, &ctn) if File.readable?(path)
 
         scratch.warn(ep, "failed to load: #{ path }")
@@ -770,6 +813,7 @@ module TypeProf
       Type::Builtin[:exc]        = scratch.get_constant(klass_obj, :Exception)
       Type::Builtin[:encoding]   = scratch.get_constant(klass_obj, :Encoding)
       Type::Builtin[:enumerator] = scratch.get_constant(klass_obj, :Enumerator)
+      Type::Builtin[:kernel]     = scratch.get_constant(klass_obj, :Kernel)
 
       klass_vmcore = Type::Builtin[:vmcore]
       klass_ary    = Type::Builtin[:ary]
@@ -782,6 +826,7 @@ module TypeProf
       scratch.set_custom_method(klass_vmcore, :"core#undef_method", Builtin.method(:vmcore_undef_method))
       scratch.set_custom_method(klass_vmcore, :"core#hash_merge_kwd", Builtin.method(:vmcore_hash_merge_kwd))
       scratch.set_custom_method(klass_vmcore, :"core#raise", Builtin.method(:vmcore_raise))
+
       scratch.set_custom_method(klass_vmcore, :lambda, Builtin.method(:lambda))
       scratch.set_singleton_custom_method(klass_obj, :"new", Builtin.method(:object_s_new))
       scratch.set_custom_method(klass_obj, :p, Builtin.method(:kernel_p), false)
@@ -804,9 +849,12 @@ module TypeProf
       scratch.set_custom_method(klass_module, :public, Builtin.method(:module_public), false)
       scratch.set_custom_method(klass_module, :private, Builtin.method(:module_private), false)
       scratch.set_custom_method(klass_module, :define_method, Builtin.method(:module_define_method))
-      scratch.set_custom_method(klass_module, :"attr_accessor", Builtin.method(:module_attr_accessor))
-      scratch.set_custom_method(klass_module, :"attr_reader", Builtin.method(:module_attr_reader))
-      scratch.set_custom_method(klass_module, :"attr_writer", Builtin.method(:module_attr_writer))
+      scratch.set_custom_method(klass_module, :attr_accessor, Builtin.method(:module_attr_accessor))
+      scratch.set_custom_method(klass_module, :attr_reader, Builtin.method(:module_attr_reader))
+      scratch.set_custom_method(klass_module, :attr_writer, Builtin.method(:module_attr_writer))
+      scratch.set_custom_method(klass_module, :class_eval, Builtin.method(:object_module_eval))
+      scratch.set_custom_method(klass_module, :module_eval, Builtin.method(:object_module_eval))
+      scratch.set_custom_method(klass_module, :===, Builtin.method(:module_eqq))
 
       scratch.set_custom_method(klass_proc, :[], Builtin.method(:proc_call))
       scratch.set_custom_method(klass_proc, :call, Builtin.method(:proc_call))
@@ -834,6 +882,12 @@ module TypeProf
       str_ty = Type::Instance.new(Type::Builtin[:str])
       env_ty = Type.gen_hash {|h| h[str_ty] = Type.optional(str_ty) }
       scratch.add_constant(klass_obj, :ENV, env_ty, false)
+
+      scratch.search_method(Type::Builtin[:kernel], false, :sprintf) do |mdefs,|
+        mdefs.each do |mdef|
+          scratch.add_method(klass_vmcore, :"core#sprintf", false, mdef)
+        end
+      end
     end
   end
 end
